﻿using Microsoft.Extensions.Logging;
using SoftwareInstallationContracts.BindingModels;
using SoftwareInstallationContracts.BusinessLogicContracts;
using SoftwareInstallationContracts.SearchModels;
using SoftwareInstallationContracts.ViewModels;
using SoftwareInstallationDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SoftwareInstallationView
{
    public partial class FormShop : Form
    {
        private readonly List<ShopViewModel>? _listShops;
        private readonly IShopLogic _logic;
        private readonly ILogger _logger;
        private int? _id;
        private Dictionary<int, (IPackageModel, int)> _listStores;
        public int Id { set { _id = value; } }

        public FormShop(ILogger<FormShop> logger, IShopLogic logic)
        {
            InitializeComponent();
            _logger = logger;
            _listShops = new();
            _logic = logic;
        }

        private void SaveButton_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(nameTextBox.Text))
            {
                MessageBox.Show("Заполните название", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (string.IsNullOrEmpty(addressTextBox.Text))
            {
                MessageBox.Show("Заполните адрес", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Сохранение магазина");

            try
            {
                var model = new ShopBindingModel
                {
                    Id = _id ?? 0,
                    ShopName = nameTextBox.Text,
                    ShopAddress = addressTextBox.Text,
                    OpeningDate = openingDatePicker.Value.Date,
                    PackageMaxCount = (int)numericUpDown.Value,
                    ShopPackages = _listStores
				};
                var operationResult = _id.HasValue ? _logic.Update(model) : _logic.Create(model);
                if (!operationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }

                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения магазина");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void FormShop_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                _logger.LogInformation("Загрузка магазина");
                try
                {
                    var view = _logic.ReadElement(new ShopSearchModel
                    {
                        Id = _id.Value
                    });
                    if (view != null)
                    {
                        nameTextBox.Text = view.ShopName;
                        addressTextBox.Text = view.ShopAddress;
                        openingDatePicker.Text = view.OpeningDate.ToString();
                        numericUpDown.Value = view.PackageMaxCount;
                        _listStores = view.ShopPackages ?? new Dictionary<int, (IPackageModel, int)>();
                        LoadData();
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка загрузки магазина");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
            }
        }
        private void LoadData(bool extendDate = true)
        {
            _logger.LogInformation("Загрузка изделий магазина");
            try
            {
                if (_listStores != null)
                {
                    dataGridView.Rows.Clear();
                    foreach (var elem in _listStores)
                    {
                        dataGridView.Rows.Add(new object[] { elem.Value.Item1.PackageName, elem.Value.Item1.Price, elem.Value.Item2 });
                    }
                }
                _logger.LogInformation("Загрузка магазинов");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки изделий магазина");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
