﻿using Microsoft.EntityFrameworkCore;
using SoftwareInstallationContracts.BindingModels;
using SoftwareInstallationContracts.SearchModels;
using SoftwareInstallationContracts.StoragesContracts;
using SoftwareInstallationContracts.ViewModels;
using SoftwareInstallationDatabaseImplement.Models;
using SoftwareInstallationDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SoftwareInstallationDatabaseImplement.Implements
{
	public class ShopStorage : IShopStorage
	{
		public ShopViewModel? Delete(ShopBindingModel model)
		{
			using var context = new SoftwareInstallationDatabase();

			var element = context.Shops
				.Include(x => x.Packages)
				.FirstOrDefault(rec => rec.Id == model.Id);

			if (element != null)
			{
				context.Shops.Remove(element);
				context.SaveChanges();

				return element.GetViewModel;
			}

			return null;
		}

		public ShopViewModel? GetElement(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
			{
				return null;
			}

			using var context = new SoftwareInstallationDatabase();

			return context.Shops
				.Include(x => x.Packages)
				.ThenInclude(x => x.Package)
				.FirstOrDefault(x => (!string.IsNullOrEmpty(model.ShopName) && x.ShopName == model.ShopName) || (model.Id.HasValue && x.Id == model.Id))
				?.GetViewModel;
		}

		public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.ShopName))
			{
				return new();
			}

			using var context = new SoftwareInstallationDatabase();

			return context.Shops
					.Include(x => x.Packages)
					.ThenInclude(x => x.Package)
					.Where(x => x.ShopName.Contains(model.ShopName))
					.ToList()
					.Select(x => x.GetViewModel)
					.ToList();
		}

		public List<ShopViewModel> GetFullList()
		{
			using var context = new SoftwareInstallationDatabase();
			return context.Shops
					.Include(x => x.Packages)
					.ThenInclude(x => x.Package)
					.ToList()
					.Select(x => x.GetViewModel)
					.ToList();
		}

		public ShopViewModel? Insert(ShopBindingModel model)
		{
			using var context = new SoftwareInstallationDatabase();
			var newShop = Shop.Create(context, model);
			if (newShop == null)
			{
				return null;
			}
			context.Shops.Add(newShop);
			context.SaveChanges();
			return newShop.GetViewModel;
		}

		public bool SellPackage(IPackageModel model, int quantity)
		{
			using var context = new SoftwareInstallationDatabase();

			if (context.Shops
					.Include(x => x.Packages)
					.ThenInclude(x => x.Package)
					.ToList()
					.Where(x => x.ShopPackages.ContainsKey(model.Id)).Select(x => x.PackageMaxCount).Sum() < quantity)
			{
				return false;
			}

			using var transaction = context.Database.BeginTransaction();
			try
			{
				foreach (var Shop in context.Shops
					.Include(x => x.Packages)
					.ThenInclude(x => x.Package)
					.ToList()
					.Where(x => x.ShopPackages.ContainsKey(model.Id)))
				{
					int countInCurrentShop = Shop.ShopPackages[model.Id].Item2;
					if (countInCurrentShop <= quantity)
					{
						var elem = context.ShopPackages
							.Where(x => x.PackageId == model.Id)
							.FirstOrDefault(x => x.ShopId == Shop.Id);
						context.ShopPackages.Remove(elem);
						Shop.ShopPackages.Remove(model.Id);
						quantity -= countInCurrentShop;
					}
					else
					{
						Shop.ShopPackages[model.Id] = (Shop.ShopPackages[model.Id].Item1, countInCurrentShop - quantity);
						quantity = 0;
						Shop.UpdatePackage(context, new()
						{
							Id = Shop.Id,
							ShopPackages = Shop.ShopPackages,
						});
					}
					if (quantity == 0)
					{
						context.SaveChanges();
						transaction.Commit();
						return true;
					}
				}
				transaction.Rollback();
				return false;
			}
			catch
			{
				transaction.Rollback();
				throw;
			}
		}

		public ShopViewModel? Update(ShopBindingModel model)
		{
			using var context = new SoftwareInstallationDatabase();
			using var transaction = context.Database.BeginTransaction();
			try
			{
				var Shop = context.Shops.FirstOrDefault(rec => rec.Id == model.Id);
				if (Shop == null)
				{
					return null;
				}
				Shop.Update(model);
				Shop.UpdatePackage(context, model);
				context.SaveChanges();
				transaction.Commit();
				return Shop.GetViewModel;
			}
			catch
			{
				transaction.Rollback();
				throw;
			}
		}
	}
}
