﻿using SoftwareInstallationDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations.Schema;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SoftwareInstallationContracts.BindingModels;
using SoftwareInstallationContracts.ViewModels;

namespace SoftwareInstallationDatabaseImplement.Models
{
	public class Shop : IShopModel
	{
		[Required]
		public string ShopName { get; private set; } = string.Empty;
		[Required]
		public string ShopAddress { get; private set; } = string.Empty;
		[Required]
		public DateTime OpeningDate { get; private set; }
		[Required]
		public int PackageMaxCount { get; private set; }

		public int Id { get; private set; }


		private Dictionary<int, (IPackageModel, int)> _ShopPackages = null;
		[NotMapped]
		public Dictionary<int, (IPackageModel, int)> ShopPackages
		{
			get
			{
				if (_ShopPackages == null)
				{
					_ShopPackages = Packages
							.ToDictionary(recPC => recPC.PackageId, recPC => (recPC.Package as IPackageModel, recPC.Count));
				}
				return _ShopPackages;
			}
		}

		[ForeignKey("ShopId")]
		public virtual List<ShopPackage> Packages { get; set; } = new();

		public static Shop Create(SoftwareInstallationDatabase context, ShopBindingModel model)
		{
			return new Shop()
			{
				Id = model.Id,
				ShopName = model.ShopName,
				ShopAddress = model.ShopAddress,
				OpeningDate = model.OpeningDate,
				PackageMaxCount = model.PackageMaxCount,
				Packages = model.ShopPackages.Select(x => new ShopPackage
				{
					Package = context.Packages.First(y => y.Id == x.Key),
					Count = x.Value.Item2
				}).ToList()
			};
		}

		public void Update(ShopBindingModel model)
		{
			ShopName = model.ShopName;
			ShopAddress = model.ShopAddress;
			OpeningDate = model.OpeningDate;
			PackageMaxCount = model.PackageMaxCount;
		}

		public ShopViewModel GetViewModel => new()
		{
			Id = Id,
			ShopName = ShopName,
			ShopAddress = ShopAddress,
			OpeningDate = OpeningDate,
			PackageMaxCount = PackageMaxCount,
			ShopPackages = ShopPackages
		};

		public void UpdatePackage(SoftwareInstallationDatabase context, ShopBindingModel model)
		{
			var ShopPackages = context.ShopPackages.Where(rec => rec.ShopId == model.Id).ToList();
			if (ShopPackages != null && ShopPackages.Count > 0)
			{   // удалили те, которых нет в модели
				context.ShopPackages.RemoveRange(ShopPackages.Where(rec => !model.ShopPackages.ContainsKey(rec.PackageId)));
				context.SaveChanges();
				// обновили количество у существующих записей
				foreach (var updatePackage in ShopPackages)
				{
					updatePackage.Count = model.ShopPackages[updatePackage.PackageId].Item2;
					model.ShopPackages.Remove(updatePackage.PackageId);
				}
				context.SaveChanges();
			}
			var Shop = context.Shops.First(x => x.Id == Id);
			foreach (var sp in model.ShopPackages)
			{
				context.ShopPackages.Add(new ShopPackage
				{
					Shop = Shop,
					Package = context.Packages.First(x => x.Id == sp.Key),
					Count = sp.Value.Item2
				});
				context.SaveChanges();
			}
			_ShopPackages = null;
		}
	}
}
