﻿using SoftwareInstallationContracts.BindingModels;
using SoftwareInstallationContracts.SearchModels;
using SoftwareInstallationContracts.StoragesContracts;
using SoftwareInstallationContracts.ViewModels;
using SoftwareInstallationDataModels.Models;
using SoftwareInstallationFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static System.Formats.Asn1.AsnWriter;

namespace SoftwareInstallationFileImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataFileSingleton source;

        public ShopStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public ShopViewModel? Delete(ShopBindingModel model)
        {
            var element = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                source.Shops.Remove(element);
                source.SaveShops();
                return element.GetViewModel;
            }
            return null;
        }

        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
            {
                return null;
            }
            return source.Shops.FirstOrDefault(x =>
                (!string.IsNullOrEmpty(model.ShopName) && x.ShopName ==
                model.ShopName) || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName))
            {
                return new();
            }
            return source.Shops.Where(x =>
                x.ShopName.Contains(model.ShopName)).Select(x => x.GetViewModel).ToList();
        }

        public List<ShopViewModel> GetFullList()
        {
            return source.Shops.Select(x => x.GetViewModel).ToList();
        }

        public ShopViewModel? Insert(ShopBindingModel model)
        {
            model.Id = source.Shops.Count > 0 ? source.Shops.Max(x => x.Id) + 1 : 1;
            var newShop = Shop.Create(model);
            if (newShop == null)
            {
                return null;
            }
            source.Shops.Add(newShop);
            source.SaveShops();
            return newShop.GetViewModel;
        }

        public bool SellPackage(IPackageModel model, int quantity)
        {
            if (source.Shops.Select(x => x.ShopPackages.FirstOrDefault(y => y.Key == model.Id).Value.Item2).Sum() < quantity)
            {
                return false;
            }
            foreach (var Shop in source.Shops.Where(x => x.ShopPackages.ContainsKey(model.Id)))
            {
                int QuantityInCurrentShop = Shop.ShopPackages[model.Id].Item2;
                if (QuantityInCurrentShop <= quantity)
                {
                    Shop.ShopPackages.Remove(model.Id);
                    quantity -= QuantityInCurrentShop;
                }
                else
                {
                    Shop.ShopPackages[model.Id] = (Shop.ShopPackages[model.Id].Item1, QuantityInCurrentShop - quantity);
                    quantity = 0;
                }
                if (quantity == 0)
                {
                    return true;
                }
            }
            return false;
        }

        public ShopViewModel? Update(ShopBindingModel model)
        {
            var Shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (Shop == null)
            {
                return null;
            }
            Shop.Update(model);
            source.SaveShops();
            return Shop.GetViewModel;
        }
    }
}
