﻿using SoftwareInstallationContracts.BindingModels;
using SoftwareInstallationContracts.ViewModels;
using SoftwareInstallationDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace SoftwareInstallationFileImplement.Models
{
    public class Shop : IShopModel
    {
        public string ShopName { get; private set; } = string.Empty;
        public string ShopAddress { get; private set; } = string.Empty;

        public DateTime OpeningDate { get; private set; }
        public Dictionary<int, int> Packages { get; private set; } = new();

        public Dictionary<int, (IPackageModel, int)> _Packages = null;
        public Dictionary<int, (IPackageModel, int)> ShopPackages
        {
            get
            {
                if (_Packages == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _Packages = Packages.ToDictionary(x => x.Key, y => ((source.Packages.FirstOrDefault(z => z.Id == y.Key) as IPackageModel)!, y.Value));
                }
                return _Packages;
            }
        }

        public int Id { get; private set; }

        public int PackageMaxCount { get; private set; }

        public static Shop? Create(ShopBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                ShopAddress = model.ShopAddress,
                PackageMaxCount = model.PackageMaxCount,
                OpeningDate = model.OpeningDate,
                Packages = model.ShopPackages.ToDictionary(x => x.Key, x => x.Value.Item2)
            };
        }
        public static Shop? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ShopName = element.Element("ShopName")!.Value,
                ShopAddress = element.Element("ShopAddress")!.Value,
                OpeningDate = Convert.ToDateTime(element.Element("OpeningDate")!.Value),
                PackageMaxCount = Convert.ToInt32(element.Element("PackageMaxCount")!.Value),
                Packages = element.Element("ShopPackages")!.Elements("Package").ToDictionary(
                    x => Convert.ToInt32(x.Element("Key")?.Value),
                    x => Convert.ToInt32(x.Element("Value")?.Value))
            };
        }
        public void Update(ShopBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            ShopName = model.ShopName;
            ShopAddress = model.ShopAddress;
            OpeningDate = model.OpeningDate;
            PackageMaxCount = model.PackageMaxCount;
            Packages = model.ShopPackages.ToDictionary(x => x.Key, x => x.Value.Item2);
            _Packages = null;
        }
        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            ShopAddress = ShopAddress,
            ShopPackages = ShopPackages,
            OpeningDate = OpeningDate,
            PackageMaxCount = PackageMaxCount,
        };
        public XElement GetXElement => new("Shop",
             new XAttribute("Id", Id),
             new XElement("ShopName", ShopName),
             new XElement("ShopAddress", ShopAddress),
             new XElement("OpeningDate", OpeningDate),
             new XElement("PackageMaxCount", PackageMaxCount),
             new XElement("ShopPackages", Packages
                 .Select(x => new XElement("Package",
                     new XElement("Key", x.Key),
                     new XElement("Value", x.Value))
                 ).ToArray()));
    }
}
